package com.freshcodes.red5pro;

import android.content.Context;
import android.hardware.Camera;
import android.view.Surface;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.WindowManager;

import androidx.annotation.NonNull;

import com.red5pro.streaming.R5Connection;
import com.red5pro.streaming.R5Stream;
import com.red5pro.streaming.config.R5Configuration;
import com.red5pro.streaming.source.R5Camera;
import com.red5pro.streaming.source.R5Microphone;

import java.io.IOException;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;

public class Red5StreamView implements PlatformView {
    @NonNull
    private final R5Configuration configuration;
    int height, width;
    int bitrate;
    int fps;

    private final SurfaceView surfaceView;
    private SurfaceHolder surfaceHolder;
    private Camera camera;

    private int cameraId = Camera.CameraInfo.CAMERA_FACING_FRONT;

    protected boolean isPublishing = false;
    String streamName;
    int recordType;
    protected R5Camera r5Camera;
    protected R5Stream stream;

    Red5StreamView(@NonNull final Context context, BinaryMessenger messenger, int id, @NonNull R5Configuration configuration, int height, int width, int bitrate, int fps) {
        this.configuration = configuration;
        this.height = height;
        this.width = width;
        this.bitrate = bitrate;
        this.fps = fps;
        surfaceView = new SurfaceView(context);
        camera = Camera.open(cameraId);
        camera.setDisplayOrientation(getCameraDisplayOrientation(context, cameraId));
        surfaceView.getHolder().addCallback(new SurfaceHolder.Callback() {
            @Override
            public void surfaceCreated(@NonNull SurfaceHolder holder) {
                try {
                    surfaceHolder = holder;
                    camera.setPreviewDisplay(surfaceHolder);
                    camera.startPreview();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void surfaceChanged(@NonNull SurfaceHolder holder, int format, int width, int height) {

            }

            @Override
            public void surfaceDestroyed(@NonNull SurfaceHolder holder) {

            }
        });

        MethodChannel methodChannel = new MethodChannel(messenger, "Red5StreamView/" + id);
        methodChannel.setMethodCallHandler(new MethodChannel.MethodCallHandler() {
            @Override
            public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
                switch (call.method) {
                    case "switchCamera":
                        switchCamera(context);
                        result.success(true);
                        break;
                    case "isPublishing":
                        result.success(isPublishing());
                        break;
                    case "startStreaming":
                        String streamKey = call.hasArgument("streamKey") ? (String) call.argument("streamKey") : null;
                        Integer recordType = call.hasArgument("recordType") ? (Integer) call.argument("recordType") : null;
                        if (streamKey != null && recordType != null) {
                            if (!isPublishing) {
                                startStreaming(streamKey, recordType);
                                result.success(true);
                            } else {
                                result.error("ALREADY_PUBLISHING", "Already stream published", "");
                            }
                        } else {
                            result.error("INVALID_PARAMETER", "StreamKey and Type should not be null", "");
                        }
                        break;
                    case "stopStreaming":
                        if (isPublishing) {
                            stopStreaming();
                            result.success(true);
                        } else {
                            result.error("NOT_LIVE_STREAM", "Stream is not live", "");
                        }
                        break;
                    default:
                        result.notImplemented();
                        break;
                }
            }
        });
    }

    @Override
    public View getView() {
        return surfaceView;
    }

    @Override
    public void dispose() {
        camera.stopPreview();
        camera.release();
    }

    public boolean isPublishing() {
        return isPublishing;
    }

    public void switchCamera(Context context) {
        camera.stopPreview();
        camera.release();
        if (cameraId == Camera.CameraInfo.CAMERA_FACING_BACK) {
            cameraId = Camera.CameraInfo.CAMERA_FACING_FRONT;
        } else {
            cameraId = Camera.CameraInfo.CAMERA_FACING_BACK;
        }
        camera = Camera.open(cameraId);
        camera.setDisplayOrientation(getCameraDisplayOrientation(context, cameraId));
        try {
            camera.setPreviewDisplay(surfaceHolder);
        } catch (IOException e) {
            e.printStackTrace();
        }
        if (isPublishing) {
            r5Camera.setCamera(camera);
            if (cameraId == Camera.CameraInfo.CAMERA_FACING_BACK) {
                r5Camera.setOrientation(90);
            } else {
                r5Camera.setOrientation(270);
            }
        }
        camera.startPreview();
    }

    public int getCameraDisplayOrientation(Context context, int cameraId) {
        android.hardware.Camera.CameraInfo info =
                new android.hardware.Camera.CameraInfo();
        android.hardware.Camera.getCameraInfo(cameraId, info);
        WindowManager windowManager = (WindowManager) context
                .getSystemService(Context.WINDOW_SERVICE);
        int rotation = windowManager.getDefaultDisplay()
                .getRotation();
        int degrees = 0;
        switch (rotation) {
            case Surface.ROTATION_0:
                degrees = 0;
                break;
            case Surface.ROTATION_90:
                degrees = 90;
                break;
            case Surface.ROTATION_180:
                degrees = 180;
                break;
            case Surface.ROTATION_270:
                degrees = 270;
                break;
        }

        int result;
        if (info.facing == Camera.CameraInfo.CAMERA_FACING_FRONT) {
            result = (info.orientation + degrees) % 360;
            result = (360 - result) % 360;
        } else {
            result = (info.orientation - degrees + 360) % 360;
        }
        return result;
    }

    public void startStreaming(String streamName, int recordType) {
        this.streamName = streamName;
        this.recordType = recordType;

        camera.stopPreview();

        stream = new R5Stream(new R5Connection(configuration));
        stream.setView(surfaceView);

        r5Camera = new R5Camera(camera, width, height);
        if (cameraId == Camera.CameraInfo.CAMERA_FACING_BACK) {
            r5Camera.setOrientation(90);
        } else {
            r5Camera.setOrientation(270);
        }
        r5Camera.setBitrate(bitrate);
        r5Camera.setFramerate(fps);
        R5Microphone r5Microphone = new R5Microphone();

        stream.attachCamera(r5Camera);
        stream.attachMic(r5Microphone);

        stream.publish(streamName, intToRecordType(recordType));
        camera.startPreview();
        isPublishing = true;
    }

    public void stopStreaming() {
        if (stream != null) {
            stream.stop();
            camera.startPreview();
        }
        isPublishing = false;
    }

    private R5Stream.RecordType intToRecordType(int type) {
        switch (type) {
            case 0:
                return R5Stream.RecordType.Live;
            case 1:
                return R5Stream.RecordType.Record;
            case 2:
                return R5Stream.RecordType.Append;
            default:
                return R5Stream.RecordType.Live;
        }
    }
}
